<?php
/**
 * @brief		Surly Behaviour
 * @author		<a href='http://sur.ly'>Sur.ly</a>
 * @copyright	(c) 2016 Sur.ly
 * @license		GNU General Public License, version 2 (GPL-2.0)
 * @package		IPS Community Suite
 * @subpackage	Surly
 * @since		28 Mar 2016
 * @version
 */

namespace IPS\surly\modules\admin\surly;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Surly Controller
 */
class _surly extends \IPS\Dispatcher\Controller
{
	const SURLY_PANEL_URL = 'https://surdotly.com/settings/';

	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'surly_manage' );

		if ( !\IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, array( 'https://fonts.googleapis.com/css?family=Noto+Sans:400,400i,700,700i' ) );
			\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'css/font-awesome.min.css', 'surly', 'interface' ) );
			\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'css/surly-style.css', 'surly', 'interface' ) );
			\IPS\Output::i()->cssFiles = array_merge( \IPS\Output::i()->cssFiles, \IPS\Theme::i()->css( 'css/chosen.css', 'surly', 'interface' ) );

			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'js/jquery-1.8.2.min.js', 'surly', 'interface' ) );
			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'js/chosen.jquery.min.js', 'surly', 'interface' ) );
			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'js/jquery.icheck.min.js', 'surly', 'interface' ) );
			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'js/jquery.postMessage.min.js', 'surly', 'interface' ) );
			\IPS\Output::i()->jsFiles = array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'js/surly-script.js', 'surly', 'interface' ) );
		}

		parent::execute();
	}

	protected function manage()
	{
		if (\IPS\Settings::i()->surly_initial)
		{
			$this->settingsInitial();
		}
		else if (\IPS\Settings::i()->surly_toolbar_id)
		{
			$this->settingsPlugin();
		}
		else
		{
			$this->settingsShort();
		}
	}

	protected function settingsShort()
	{
		$surlyReplaceUrls = json_decode( \IPS\Settings::i()->surly_replace_urls, TRUE );

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'surly_tab_plugin_settings' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'admin', 'surly' )->surlySettingsShort(
			$surlyReplaceUrls
		);
	}

	protected function settingsInitial()
	{
		$surlyReplaceUrls = json_decode( \IPS\Settings::i()->surly_replace_urls, TRUE );
		$surlyTrustedDomains = json_decode( \IPS\Settings::i()->surly_trusted_domains, TRUE );
		$surlyTrustedGroups = json_decode( \IPS\Settings::i()->surly_trusted_groups, TRUE );
		$surlySubdomain = \IPS\Settings::i()->surly_subdomain ?: '';
		$surlyInitial = \IPS\Settings::i()->surly_initial ?: '1-0';
		$surlyAuthUrl = self::SURLY_PANEL_URL . 'auth/?' . http_build_query(array(
			'url' => \IPS\Settings::i()->base_url,
			'cmsId' => \IPS\surly\Surly\SurlyIpb::CMS_ID,
			'meta' => array('cms_version' => \IPS\Application::load('core')->version),
		));


		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'surly_tab_plugin_settings' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'admin', 'surly' )->surlySettingsInitial(
			$surlyReplaceUrls,
			$surlyTrustedDomains,
			$surlyTrustedGroups,
			$surlySubdomain,
			$surlyInitial,
			$surlyAuthUrl
		);
	}

	protected function settingsPlugin()
	{
		$surlyReplaceUrls = json_decode( \IPS\Settings::i()->surly_replace_urls, TRUE );
		$surlyTrustedDomains = json_decode( \IPS\Settings::i()->surly_trusted_domains, TRUE );
		$surlyTrustedGroups = json_decode( \IPS\Settings::i()->surly_trusted_groups, TRUE );
		$surlySubdomain = \IPS\Settings::i()->surly_subdomain ?: '';

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'surly_tab_plugin_settings' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'admin', 'surly' )->surlySettingsPlugin(
			$surlyReplaceUrls,
			$surlyTrustedDomains,
			$surlyTrustedGroups,
			$surlySubdomain
		);
	}

	protected function settingsToolbar()
	{
		$surlyPanelUrl = self::SURLY_PANEL_URL . \IPS\Settings::i()->surly_toolbar_id . '/plugin_access?password=' . \IPS\Settings::i()->surly_toolbar_password;

		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'surly_tab_toolbar_settings' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'admin', 'surly' )->surlySettingsToolbar(
			$surlyPanelUrl
		);
	}

	protected function saveSubdomain()
	{
		$surly_subdomain = isset( \IPS\Request::i()->surly_subdomain )
			? \IPS\Request::i()->surly_subdomain
			: '';

		if ( $surly_subdomain != \IPS\Settings::i()->surly_subdomain )
		{
			if ( $surly_subdomain == '' )
			{
				\IPS\surly\Surly\SurlyIpb::i()->trackHistory( \IPS\surly\Surly\SurlyIpb::SURLY_ACTION_TYPE_SUBDOMAIN_UNLINK );
			}
			else
			{
				$result = json_decode( \IPS\surly\Surly\SurlyIpb::i()->linkSubdomain($surly_subdomain), TRUE );

				if ( isset( $result['error_code'] ) )
				{
					$errors = array(
						1 => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_subdomain_require' ),
						2 => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_subdomain_invalid' ),
						3 => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_subdomain_exist' ),
						4 => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_subdomain_cname' ),
						5 => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_unexpected' ),
					);

					return isset( $errors[$result['error_code']] )
						? array( 'error' => $errors[$result['error_code']] )
						: array( 'error' => $errors[5] );
				}
				elseif ( isset( $result['subdomain'] ) )
				{
					$surly_subdomain = $result['subdomain'];

					\IPS\surly\Surly\SurlyIpb::i()->trackHistory( \IPS\surly\Surly\SurlyIpb::SURLY_ACTION_TYPE_SUBDOMAIN_LINK );
				}
				else
				{
					$surly_subdomain = \IPS\Settings::i()->surly_subdomain;
				}
			}
		}

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => $surly_subdomain ), array( 'conf_key=?', 'surly_subdomain' ) );

		unset( \IPS\Data\Store::i()->settings );
	}

	protected function saveTrustedGroups()
	{
		$surly_trusted_groups = isset( \IPS\Request::i()->surly_trusted_groups )
			? (array) \IPS\Request::i()->surly_trusted_groups
			: array();

		$groups = \IPS\Member\Group::groups();

		foreach ( $surly_trusted_groups as $key => $value )
		{
			if ( !isset( $groups[$value] ) )
			{
				unset( $surly_trusted_groups[$key] );
			}
		}

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => json_encode( $surly_trusted_groups ) ), array( 'conf_key=?', 'surly_trusted_groups' ) );

		unset( \IPS\Data\Store::i()->settings );
	}

	protected function saveShortenUrls()
	{
		$surly_shorten_urls = !empty( \IPS\Request::i()->surly_shorten_urls );

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => (int) $surly_shorten_urls ), array( 'conf_key=?', 'surly_shorten_urls' ) );

		unset( \IPS\Data\Store::i()->settings );
	}

	protected function saveReplaceUrls()
	{
		$surly_replace_urls = isset( \IPS\Request::i()->surly_replace_urls )
			? (array) \IPS\Request::i()->surly_replace_urls
			: array(0);

		$surly_replace_urls = array_intersect( array(0), $surly_replace_urls )
			?: array_intersect( array(1), $surly_replace_urls )
			?: array(0);

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => json_encode( $surly_replace_urls ) ), array( 'conf_key=?', 'surly_replace_urls' ) );

		unset( \IPS\Data\Store::i()->settings );
	}

	protected function ajaxSaveToolbarSettings()
	{
		$surly_toolbar_id = isset( \IPS\Request::i()->surly_toolbar_id )
			? \IPS\Request::i()->surly_toolbar_id
			: '';

		$surly_toolbar_password = isset( \IPS\Request::i()->surly_toolbar_password )
			? \IPS\Request::i()->surly_toolbar_password
			: '';

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => $surly_toolbar_id ), array( 'conf_key=?', 'surly_toolbar_id' ) );
		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => $surly_toolbar_password ), array( 'conf_key=?', 'surly_toolbar_password' ) );
		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => '2-1' ), array( 'conf_key=?', 'surly_initial' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\surly\Surly\SurlyIpb::i()->trackHistory( \IPS\surly\Surly\SurlyIpb::SURLY_ACTION_TYPE_AUTH );

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}

	protected function ajaxSaveSubdomain()
	{
		$result = $this->saveSubdomain();

		if ( isset( $result['error'] ) )
		{
			\IPS\Output::i()->json( array( 'error' => array( 'subdomain' => $result['error'] ) ) );
		}

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => '2-2' ), array( 'conf_key=?', 'surly_initial' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}

	protected function ajaxSaveTrustedGroups()
	{
		$this->saveTrustedGroups();

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => '4-1' ), array( 'conf_key=?', 'surly_initial' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}

	protected function ajaxSaveShortenUrls()
	{
		$this->saveShortenUrls();

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => '4-2' ), array( 'conf_key=?', 'surly_initial' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}

	protected function ajaxSaveReplaceUrls()
	{
		$this->saveReplaceUrls();

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => 0 ), array( 'conf_key=?', 'surly_initial' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\surly\Surly\SurlyIpb::i()->trackHistory( \IPS\surly\Surly\SurlyIpb::SURLY_ACTION_TYPE_ACTIVATION );

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}

	protected function ajaxSkipStep()
	{
		$steps = array( '1-0', '2-1', '2-2', '3-0', '4-1', '4-2' );

		$next_step = isset( \IPS\Request::i()->next_step ) && in_array( \IPS\Request::i()->next_step, $steps )
			? \IPS\Request::i()->next_step
			: $steps[0];

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => $next_step ), array( 'conf_key=?', 'surly_initial' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}

	protected function ajaxSaveSettings()
	{
		$this->saveReplaceUrls();
		$this->saveShortenUrls();
		$this->saveTrustedGroups();

		$result = $this->saveSubdomain();

		if ( isset( $result['error'] ) )
		{
			\IPS\Output::i()->json( array( 'error' => array( 'subdomain' => $result['error'] ) ) );
		}

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}

	protected function ajaxSaveTrustedDomain()
	{
		$trusted_domain = isset( \IPS\Request::i()->surly_trusted_domain )
			? preg_replace('/^https?:\/\/(.+)/',"$1", strtolower( trim( \IPS\Request::i()->surly_trusted_domain ) ))
			: '';

		if ( empty( $trusted_domain ) )
		{
			\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_domain_name' ) ) );
		}

		if ( preg_match('!(?P<host>(?:[a-z0-9_-]+\.)+[a-z]+)!u', $trusted_domain, $result) )
		{
			$trusted_domain = $result['host'];
		}
		else
		{
			\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_domain_name' ) ) );
		}

		$surlyTrustedDomains = json_decode( \IPS\Settings::i()->surly_trusted_domains, TRUE );

		if ( in_array($trusted_domain, $surlyTrustedDomains) )
		{
			\IPS\Output::i()->json( array( 'error' => \IPS\Member::loggedIn()->language()->addToStack( 'surly_error_domain_exist' ) ) );
		}

		$surlyTrustedDomains = array_merge( $surlyTrustedDomains, array( $trusted_domain ) );

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => json_encode($surlyTrustedDomains) ), array( 'conf_key=?', 'surly_trusted_domains' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\Output::i()->json( array('domain' => $trusted_domain) );
	}

	protected function ajaxDeleteTrustedDomains()
	{
		$trusted_domains = isset( \IPS\Request::i()->surly_trusted_domains )
			? (array) \IPS\Request::i()->surly_trusted_domains
			: array();

		$surlyTrustedDomains = json_decode( \IPS\Settings::i()->surly_trusted_domains, TRUE );

		$surlyTrustedDomains = array_diff( $surlyTrustedDomains, $trusted_domains );

		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => json_encode($surlyTrustedDomains) ), array( 'conf_key=?', 'surly_trusted_domains' ) );

		unset( \IPS\Data\Store::i()->settings );

		\IPS\Output::i()->json( array( 'success' => TRUE ) );
	}
}
