<?php
/**
 * @brief		Surly SDK
 * @author		<a href='http://sur.ly'>Sur.ly</a>
 * @copyright	(c) 2016 Sur.ly
 * @package		IPS Community Suite
 * @subpackage	Surly
 * @since		28 Mar 2016
 * @version
 */

namespace IPS\surly\Surly;

require_once 'surly-sdk-php/surly/Surly.php';

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Surly
 */
class _SurlyIpb extends \Surly
{
	const CMS_ID = 13;

	static protected $instance;

	protected $rootStatusKey = 'surly_root_status';
	public $isRootDomainAlive = null;

	const SURLY_ACTION_TYPE_INSTALL = 1;
	const SURLY_ACTION_TYPE_AUTH = 2;
	const SURLY_ACTION_TYPE_ACTIVATION = 3;
	const SURLY_ACTION_TYPE_DEACTIVATION = 4;
	const SURLY_ACTION_TYPE_UNINSTALL = 5;
	const SURLY_ACTION_TYPE_SUBDOMAIN_LINK = 6;
	const SURLY_ACTION_TYPE_SUBDOMAIN_UNLINK = 7;

	const SURLY_REPLACE_URLS_NOWHERE = 0;
	const SURLY_REPLACE_URLS_POSTS = 1;

	public static function i()
	{
		if (empty(static::$instance))
		{
			static::$instance = new static( \IPS\Settings::i()->surly_toolbar_id, (bool) \IPS\Settings::i()->surly_shorten_urls );

			if ( \IPS\Settings::i()->surly_subdomain )
			{
				static::$instance->setPanelHost( \IPS\Settings::i()->surly_subdomain );
			}

			foreach ( static::$instance->getTrustedDomains() as $domain )
			{
				static::$instance->whitelist($domain);
			}

			$base_url = parse_url( \IPS\Settings::i()->base_url, PHP_URL_HOST );

			static::$instance->whitelist( $base_url );
		}

		return static::$instance;
	}

	public function processByGroup($html, $groupId)
	{
		if ( in_array(static::SURLY_REPLACE_URLS_POSTS, json_decode( \IPS\Settings::i()->surly_replace_urls, TRUE ))
			&& !in_array( $groupId, $this->getTrustedGroups() ) )
		{
			return $this->process($html);
		}

		return $html;
	}

	public function getTrustedGroups()
	{
		return json_decode( \IPS\Settings::i()->surly_trusted_groups, TRUE );
	}

	public function getTrustedDomains()
	{
		return json_decode( \IPS\Settings::i()->surly_trusted_domains, TRUE );
	}

	public function cacheShortIds($url2shortIds)
	{
		foreach ( $url2shortIds as $url => $id )
		{
			\IPS\Db::i()->insert( 'surly_cache', array(
				'long_url' => $url,
				'hash' => $this->_md5($url),
				'short_id' => $id,
			), TRUE );
		}

		parent::cacheShortIds( $url2shortIds );
	}

	public function getCachedShortIds($urls)
	{
		$result = parent::getCachedShortIds($urls);

		if ( !empty( $urls ) )
		{
			foreach ( $urls as $url )
			{
				try {
					$row = \IPS\Db::i()->select( '*', 'surly_cache', array( 'hash=? and long_url=?', $this->_md5($url), $url ) )->first();

					$result[$row['long_url']] = $row['short_id'];

				} catch (\UnderflowException $e) {}
			}
		}

		return $result;
	}

	public function getCachedRootStatus()
	{
		$rootStatusKey = $this->rootStatusKey;

		if ( !empty( \IPS\Settings::i()->$rootStatusKey ) )
		{
			return (bool) \IPS\Settings::i()->$rootStatusKey;
		}

		return false;
	}

	public function cacheRootStatus($rootStatus)
	{
		\IPS\Db::i()->update( 'core_sys_conf_settings', array( 'conf_value' => (int) $rootStatus ), array( 'conf_key=?', $this->rootStatusKey ) );

		unset( \IPS\Data\Store::i()->settings );
	}

	protected function _md5($url)
	{
		$hash = md5($url);
		$result = '';

		for ($i = 0; $i < 32; $i+=2)
		{
			$digits = substr($hash, $i, 2);
			$number = hexdec($digits);
			$result.=chr($number);
		}

		return $result;
	}

	public function linkSubdomain($subdomain)
	{
		return $this->_performRequest(
			$this->apiHost . SURLY_API_SUBDOMAIN_LINK, 'POST',
			array(
				'toolbar_id' => \IPS\Settings::i()->surly_toolbar_id,
				'password' => \IPS\Settings::i()->surly_toolbar_password,
				'subdomain' => $subdomain,
			)
		);
	}

	public function trackHistory($actionType)
	{
		$this->timeout = SURLY_API_TRACK_TIMEOUT;

		$this->_performRequest(
			$this->apiHost . SURLY_API_TRACK_HISTORY_PATH,  'POST',
				array(
					'action_type' => $actionType,
					'site_url' => \IPS\Settings::i()->base_url,
					'toolbar_id' => \IPS\Settings::i()->surly_toolbar_id,
					'hash' => \IPS\Settings::i()->surly_activation_hash,
				)
		);

		$this->timeout = SURLY_API_TIMEOUT;
	}
}